package dmch;

/* 
 * DmcServer.java
 *
 * Matthew Wronka
 * Last Modified 2003-10-19
 *
 *
 * Main shell for messaging client
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 *
 */


import java.net.*;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.*;
import java.security.KeyStore;

import java.io.*;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.PrintWriter;
import java.util.StringTokenizer;

/**
 * This creates a server socket for clients to connect to.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 *
 */
public class DmcServer {
	final public static int DEFAULT_PORT = 5050;
	static protected SSLServerSocket m_SSocket = null;
	static protected DmcServerThread DST = null;
	protected final static String KEYSTORE = 
		System.getProperty("user.home")+"/.dmc/keystore";
	protected final static String PASSWORD = "password";
	protected final static String ALGORITHM= "sunx509";

	/**
	 * Creates a server socket for clients to connect to
	 *
	 * @param port Port to listen on.  If port==0, DmcServer.DEFAULT_PORT
	 *	is used.
	 */
	 static public void CreateSocket(int port) {
	 	javax.net.ssl.SSLContext SSLC = null;
		KeyManagerFactory kmf;
		KeyManager[] km;
		KeyStore ks;
		TrustManagerFactory tmf;
		TrustManager[] tm;

		try{
		ks = KeyStore.getInstance("JKS");
		ks.load(new FileInputStream(KEYSTORE), PASSWORD.toCharArray());
		kmf = KeyManagerFactory.getInstance(ALGORITHM);
		kmf.init(ks, PASSWORD.toCharArray());
		km = kmf.getKeyManagers();
		tmf = TrustManagerFactory.getInstance(ALGORITHM);
		tmf.init(ks);
		tm = tmf.getTrustManagers();
	
		SSLC = javax.net.ssl.SSLContext.getInstance("TLS");
		SSLC.init(km, tm, null);
		} catch (Exception e) { System.err.println(e); }

	 	if(port==0) port=DEFAULT_PORT;
		try {
		m_SSocket = (SSLServerSocket) (SSLC.getServerSocketFactory()
				.createServerSocket(port));
		} catch (IOException ioe) {
			System.err.println("Couldn't create server socket.");
			System.err.println(ioe);
		}

		DST = new DmcServerThread(m_SSocket);
		DST.start();
	 }

	/**
	 * Posts a new message to send to clients.
	 *
	 * @param sFormattedMessage Message to send to clients (verbatim).
	 */
	 static public void addMessage(String sFormattedMessage) {
		// Actuall forming of message is currently done
		// From within ChatUpdater/ChatLogger in TabbedConnection
		DST.addMessage(sFormattedMessage);
	 }
}

class StringList {
	boolean bSet=false;
	String sMessage=null;
	StringList next=null;
}

class DmcServerThread extends Thread {
	static protected SSLServerSocket m_SSocket = null;
	static protected java.util.LinkedList lstClients = 
			new java.util.LinkedList();
	static protected StringList slHead=new StringList(); 

	/**
	 * CTOR, sets socket to use.
	 *
	 * @param ss SSLServerSocket to use.
	 */
	public DmcServerThread(SSLServerSocket ss) {
		m_SSocket = ss;
	}

	/**
	 * Addes a message to queue to send.
	 *
	 * @param sFormattedMessage Message to send to clients (verbatim).
	 */
	public void addMessage(String sFormattedMessage) {
		java.util.ListIterator li = lstClients.listIterator(0);

		while (li.hasNext()) {
			/*
			try {
			new PrintWriter(((SSLSocket)li.next())
					.getOutputStream())
			*/
			PrintWriter t = ((PrintWriter)li.next());
			t.println(sFormattedMessage);
			t.flush();

			/*
			} catch (IOException e) {
				System.err.println("Uhh:" + e);
				lstClients.remove(li.previousIndex());
			}//*/
		}

		/*
		slHead.next=new StringList();
		slHead.sMessage=sFormattedMessage;
		slHead.bSet=true;
		*/
	}

	/**
	 * Listen for and accepts incoming connections
	 *
	 */
	 public void run() {
	 	System.err.println("Running DST");
	 	while (true) {
			SSLSocket clientSocket  = null;
			try {
				clientSocket = (SSLSocket)m_SSocket.accept();
			} catch (IOException ioe) {
				System.out.println(ioe);
			}

			if(clientSocket != null) {
			try{
				lstClients.add(new PrintWriter(clientSocket
						.getOutputStream()));
			} catch (Exception e){}
				new DmcServerConnection(slHead, clientSocket);
				
			}
			yield();	//?
		}
	}
}

class DmcServerConnection {
	private SSLSocket m_client=null;
	private StringList m_slHead = null;
	/**
	 * Sets the client's socket.
	 *
	 * @param slHead Head of the list of messages
	 * @param clientSocket Client's socket for this connection.
	 */
	DmcServerConnection(StringList slHead, SSLSocket clientSocket) {
		m_slHead=slHead;	m_client=clientSocket;

		System.err.print("New ");
		try {
			//new checkServer(m_slHead, new PrintWriter(m_client
			//	.getOutputStream())).start();
			new checkClient( new BufferedReader(
				new InputStreamReader (m_client 
					.getInputStream()))) .start();
		} catch(IOException ioe) {}
		System.err.println("Connection");
	 }
}


class checkServer extends Thread {
	protected StringList m_slHead = null;
	protected PrintWriter m_out = null;
	checkServer(StringList slHead, PrintWriter out){ 
		m_slHead=slHead;	m_out=out;
	}


	/**
	 * Gets incoming events, sends them to connected clients
	 *
	 */
	public void run() {
		m_out.println("Hello.");
		while(true) {
			if(m_slHead.bSet) {
				// Send Message and increment
				m_out.println(m_slHead.sMessage);
				m_slHead=m_slHead.next;
			} else 
				yield();
		}
	 }
}

class checkClient extends Thread {
	protected BufferedReader m_in = null;
	checkClient(BufferedReader in){ 
		m_in=in;
	}


	/**
	 * Listen for clients commands, send them to GUI
	 *
	 */
	public void run() {
		String s;

		while(true){
			try {
				s=m_in.readLine();
				if(s==null) break;
				DmcServer.addMessage(s);
				System.err.println(s);
			} catch (IOException ioe) { ; }
			yield();
		}
	 }
}
