package wtalk.protocols.dmc;

import java.io.*;
import java.net.*;
import java.util.*;

import wtalk.RMsgHandler;
import wtalk.protocols.WTalkProtocol;

public class DMC implements WTalkProtocol {
	final static int DEFAULT_PORT = 5050;

	final static String VERSION = "20030829+";
	final static String NAME = "DMC Client Protocol";
	final static String AUTHOR = "Matthew Wronka <wronkm@cs.rpi.edu>";

	protected DMCProtocol mProtocol = null;
	protected Socket us = null;
	protected  PrintWriter out = null;
	private ReceivedListener rl = null;

	private String mServer="";
	private String mName="";
	private String mPassword="";
	private int mPort=0;

	// handles displaying of messages if !null, and not overridden;
	private RMsgHandler dmh = null;  
	// handles displaying of private messages if !null;
	private RMsgHandler sph = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler smh = null;  
	// handles displaying of system messages if !null;
	private RMsgHandler rah  = null;  
	// handles DMC specific messages if !null;
	private RMsgHandler dmch = null;

	static public String getBanner() {
		return NAME + " v" + VERSION + " by " + AUTHOR;
	}

	public DMC() {
		System.err.println(getBanner());
		DMCInit();
	}

	private void DMCInit() {
		rl = new ReceivedListener();
	}

	private void resetReader() {
		rl.ReportTo(dmh);
		rl.ReportPrivateTo(sph);
		rl.ReportSystemTo(smh);
		rl.ReportAutoTo(rah);
		rl.ReportClientTo(dmch);
	}



	/***
	 *	Below we have the implementation of the
	 *	WTalkProtocol interface.
	 ***/

	//

	// Callbacks
	//
	public void setReceivedMessageHandler(RMsgHandler r) { 
		rl.ReportTo(r); 
		dmh=r;
	}	// Default
	public void setChatMessageHandler(RMsgHandler r) { rl.ReportTo(r); }	// Chats
	public void setPrivateMessageHandler(RMsgHandler r) { 
		rl.ReportPrivateTo(r); 
		sph=r;
	} // Private
	public void setWallMessageHandler(RMsgHandler r) { rl.ReportWallTo(r); }	// Walls
	public void setSystemMessageHandler(RMsgHandler r) { 
		rl.ReportSystemTo(r); 
		smh=r;
	}	// System..

	public void setAutoMessageHandler(RMsgHandler r){ 
		rl.ReportAutoTo(r); 
		rah = r;
	}

	public void setClientMessageHandler(RMsgHandler r){
		rl.ReportClientTo(r);
		dmch = r;
	}

	public void setUserListHandler(wtalk.IUserList ul) {

		// XXX: Implement Me!

	}

	//
	// Uncooked functions
	//
	public void sendRaw(String msg) { };


	//
	// Authentication/Login
	//
	public void connect() throws IOException {
		DMCInit();
		resetReader();

		us = new Socket(mServer, mPort==0? DEFAULT_PORT : mPort);

		mProtocol = new DMCProtocol(us);
		rl.setProtocol(mProtocol);
		mProtocol.Connect(mServer, mPort, mPassword, mName);

		out = new PrintWriter(us.getOutputStream(), true);
		rl.ListenTo(new BufferedReader( new InputStreamReader(us.getInputStream())));
		rl.start();
	}

	public void connect(String login) throws IOException {
		connect(mServer, mPort, login);
	}

	public void connect(String server, int port) throws IOException {
		mServer = server;	mPort = port;
		connect();
	}

	public void connect(String server, int port, String login) throws IOException {
		mName = login;
		connect(server, port);
		// send login;
	}

	public void disconnect(String msg) { 
		//if(mIRCProtocol==null) return;

		//mIRCProtocol.sendQuit(msg);

		//mIRCProtocol=null;
	}


	public boolean isConnected() { return us!=null && us.isConnected(); }

	public void setName(String name) { 
		mName = name; 
	}

	public String getName() { return mName; }

	public void setPassword(String passwd) { mPassword = passwd; }
	public void changePassword(String oldpwd, String newpwd) {  }

	public void setServer(String server) { mServer = server; }
	public String getServer() { return mServer; }
	public void setServerPort(int port) { mPort = port;}
	public int getServerPort() { return mPort; }



	//
	// IM functions
	//
	public void sendMessage(String to, String data) {
	//	if(mIRCProtocol==null) return;

		// mIRCProtocol.sendPrivMsg(to, data);
	}

	public void whois(String user) { 
	}

	public void addBuddy(String buddy, String group){}
	public void removeBuddy(String buddy, String group){}
	//listBuddies();

	public void setAway(String message){}

	//addDeny();
	//removeDeny();


	//
	// Chat functions
	//
	public void joinChat(String room) { }

	public void partChat(String room) { }

	public void chatMessage(String room, String data) { };
	public void setTopic(String room, String topic) { }

	public void chatList(String room) { }


	//
	// Admin functions
	//
	public void sendWall(String msg) {	};

	public void sendSub(String to, String data){ }

	public void setBan(String user, boolean banned){}
}
